﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;

namespace WpfMapTest
{
    public partial class MainWindow : Window
    {
        int MyImageSize = 20;
        BitmapSource MyBitmapSource; 
        public MainWindow()
        {
            InitializeComponent();
            MyImageSize = (int)SliderImSize.Value;
            var random = new Random();
            var pixels = new byte[MyImageSize * MyImageSize * 4];
            random.NextBytes(pixels);
            MyBitmapSource = BitmapSource.Create(MyImageSize, MyImageSize, 96, 96, PixelFormats.Pbgra32, null, pixels, MyImageSize * 4);
            ImageMap.Source = MyBitmapSource;
            RenderOptions.SetBitmapScalingMode(ImageMap, BitmapScalingMode.NearestNeighbor);
            
        }
        private void Window_Closed(object sender, EventArgs e)
        {

        }
        /// <summary>
        /// Creates and apllies random image source.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BtnNew_Click(object sender, RoutedEventArgs e)
        {
            var random = new Random();
            var pixels = new byte[MyImageSize * MyImageSize * 4];
            random.NextBytes(pixels);
            MyBitmapSource = BitmapSource.Create(MyImageSize, MyImageSize, 96, 96, PixelFormats.Pbgra32, null, pixels, MyImageSize * 4);
            ImageMap.Source = MyBitmapSource;
        }

        private void ImageMap_MouseWheel(object sender, MouseWheelEventArgs e)
        {
            /// <summary>
            /// Item1: x, Item2: y, Item3: theta
            /// </summary>
            var Route = new List<Tuple<int, int, double>>();
            Route.Add(new Tuple<int, int, double>(320, 240, 180));
            Route.Add(new Tuple<int, int, double>(639, 479, 4.32));
            foreach (Tuple<int, int, double> t in Route)
                System.Diagnostics.Debug.WriteLine("x: " + t.Item1 + " y: " + t.Item2 + " theta: " + t.Item3);
    }

        private void BtnDraw_Click(object sender, RoutedEventArgs e)
        {
            int stride = MyBitmapSource.PixelWidth * 4;
            int size = MyBitmapSource.PixelHeight * stride;
            byte[] pixels = new byte[size];
            int startPixel = 0;
            int line = 0;
            var random = new Random();
            line = random.Next(MyImageSize - 1);
            LabelStatus.Content = line + ". line choosen.";
            startPixel =  line * 4 * MyImageSize;
            MyBitmapSource.CopyPixels(pixels, stride, 0);
            // random sort feketére 
            try
            {
                for (int i = startPixel; i < MyImageSize * 4 + startPixel; i += 4)
                {
                    pixels[i] = 0; // r
                    pixels[i + 1] = 0; // g 
                    pixels[i + 2] = 0; // b 
                    pixels[i + 3] = 255; // alpha 
                }

                MyBitmapSource = BitmapSource.Create(MyImageSize, MyImageSize, 96, 96, PixelFormats.Pbgra32, null, pixels, MyImageSize * 4);
                ImageMap.Source = MyBitmapSource;
                LabelStatus.Background = Brushes.DarkMagenta; //.MediumSeaGreen;
            }
            catch(Exception ex)
            {
                LabelStatus.Background = Brushes.Firebrick;
                LabelStatus.Content = ex.ToString();
            }
        }

        private void BtnRenderMode_Click(object sender, RoutedEventArgs e)
        {
            BitmapScalingMode bitmapScalingMode = RenderOptions.GetBitmapScalingMode(ImageMap);
            if (bitmapScalingMode != BitmapScalingMode.NearestNeighbor)
            {
                RenderOptions.SetBitmapScalingMode(ImageMap, BitmapScalingMode.NearestNeighbor);
                LabelStatus.Content = "BitmapScalingMode is NearestNeighbor";
            }
            else
            {
                RenderOptions.SetBitmapScalingMode(ImageMap, BitmapScalingMode.LowQuality);
                LabelStatus.Content = "BitmapScalingMode is LowQuality";
            }
        }

        private void SliderImSize_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            MyImageSize = (int)SliderImSize.Value;
            LabelStatus.Content = "New image size is: " + MyImageSize + " by " + MyImageSize + ".";
        }
        /// <summary>
        /// "Drawing" on the map
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ImageMap_MouseMove(object sender, MouseEventArgs e)
        {
            if (Mouse.LeftButton == MouseButtonState.Pressed)
            {
                try {
                    int Xpixel = (int)(e.GetPosition(ImageMap).X / ImageMap.ActualWidth * MyImageSize);
                    int Ypixel = Ypixel = (int)(e.GetPosition(ImageMap).Y / ImageMap.ActualHeight * MyImageSize);
                    int stride = MyBitmapSource.PixelWidth * 4;
                    int size = MyBitmapSource.PixelHeight * stride;
                    byte[] pixels = new byte[size];
                    MyBitmapSource.CopyPixels(pixels, stride, 0);
                    int Pixel = Ypixel * 4 * MyImageSize + Xpixel * 4;
                    pixels[Pixel] = 0; // r
                    pixels[Pixel + 1] = 0; // g 
                    pixels[Pixel + 2] = 0; // b 
                    pixels[Pixel + 3] = 255; // alpha 
                    MyBitmapSource = BitmapSource.Create(MyImageSize, MyImageSize, 96, 96, PixelFormats.Pbgra32, null, pixels, MyImageSize * 4);
                    ImageMap.Source = MyBitmapSource;
                    LabelStatus.Content = "x: " + Xpixel + " y: " + Ypixel + " is colored.";
                    LabelStatus.Background = Brushes.MediumSeaGreen;
                }
                catch(Exception ex)
                {
                    LabelStatus.Background = Brushes.Firebrick;
                    LabelStatus.Content = ex.ToString();
                }
            }
            else
            {
                LabelStatus.Background = Brushes.DarkMagenta;
            }
        }
    }
}
